==> aim-transport-0.8.0.tar.gz: aim-transport-0.8.0/src/s10n.c <==
#include "aimtrans.h"

int at_server_s10n(jpacket jp)
{
    /* These are basically functions to tell you that you're stupid */
    log_debug(ZONE, "Handling server subscription.");
    switch(jpacket_subtype(jp))
    {
        case JPACKET__SUBSCRIBED:
        case JPACKET__UNSUBSCRIBED:
            /* XXX: Do what for these? */
        case JPACKET__UNSUBSCRIBE:
        case JPACKET__SUBSCRIBE:
            /* XXX:  You must have a session to do both of these */
            jutil_error(jp->x, TERROR_REGISTER);
            ehandler_send(ehandler_at, jp->x, jp->from->server);
            return 1;
        default:
            jutil_error(jp->x, TERROR_NOTACCEPTABLE);
            ehandler_send(ehandler_at, jp->x, jp->from->server);
            return 1;
    }
    return 0;
}

int at_session_s10n(jpacket jp)
{
    session s;
    xmlnode x;

    log_debug(ZONE, "Handling session subscription");
    switch(jpacket_subtype(jp))
    {
        case JPACKET__SUBSCRIBE:
            if(jp->to->resource == NULL)
                return at_buddy_add(jp);
            return 0;
        case JPACKET__UNSUBSCRIBE:
            /*XXX Do more with this 
            * (Actually take the user off the aim roster)
            * should we check for a subscription as well? 
            */
            s=at_session_find_by_jid(jp->from);
            if(s==NULL)
            {
                x = xmlnode_dup(jp->x);
                jutil_error(x, TERROR_REGISTER);
                ehandler_send(ehandler_at, x, jp->from->server);
                return 0;
            }
            at_session_buddy_unlink(s, jp->to->user);
            aim_remove_buddy(&s->ass, aim_getconn_type(&s->ass,
                            AIM_CONN_TYPE_BOS), jp->to->user);
            log_debug(ZONE, "[AIM] Unsubscribing\n");
            x = xmlnode_dup(jp->x);
            xmlnode_put_attrib(x, "type", "unsubscribed");
            xmlnode_put_attrib(x, "to", jid_full(jp->from));
            xmlnode_hide_attrib(x, "from");
            ehandler_send(ehandler_at, x, jp->from->server);
            return 0;
        /* Do I need to do anything? */
        case JPACKET__SUBSCRIBED:
        case JPACKET__UNSUBSCRIBED:
        default:
            return 0;
    }
    return 0;
}

==> aim-transport-0.9.0.tar.gz: aim-transport-0.9.0/src/s10n.c <==
#include "aimtrans.h"

int at_server_s10n(ati ti, jpacket jp)
{
    /* These are basically functions to tell you that you're stupid */
    log_debug(ZONE, "Handling server subscription.");
    switch(jpacket_subtype(jp))
    {
        case JPACKET__SUBSCRIBED:
        case JPACKET__UNSUBSCRIBED:
            /* XXX: Do what for these? */
        case JPACKET__UNSUBSCRIBE:
        case JPACKET__SUBSCRIBE:
            /* XXX:  You must have a session to do both of these */
            jutil_error(jp->x, TERROR_REGISTER);
            deliver(dpacket_new(jp->x), ti->i);
            return 1;
        default:
            jutil_error(jp->x, TERROR_NOTACCEPTABLE);
            deliver(dpacket_new(jp->x), ti->i);
            return 1;
    }
    return 0;
}

int at_session_s10n(at_session s, jpacket jp)
{
    ati ti;
    xmlnode x;

    ti = s->ti;

    log_debug(ZONE, "Handling session subscription");
    switch(jpacket_subtype(jp))
    {
        case JPACKET__SUBSCRIBE:
            if(jp->to->resource == NULL)
            {
                return at_buddy_add(ti, jp);
            } else {
                x = jutil_presnew(JPACKET__SUBSCRIBED, jid_full(jp->from), NULL);
                jutil_tofrom(x);
                xmlnode_put_attrib(x, "from", jid_full(jp->to));
                deliver(dpacket_new(x), ti->i);
                return 0;
            }
        case JPACKET__UNSUBSCRIBE:
            /*XXX Do more with this 
            * (Actually take the user off the aim roster)
            * should we check for a subscription as well? 
            */
            xhash_zap(s->buddies, jp->to->user);
            aim_remove_buddy(s->ass, aim_getconn_type(s->ass,
                            AIM_CONN_TYPE_BOS), jp->to->user);
            log_debug(ZONE, "[AIM] Unsubscribing\n");
            x = jutil_presnew(JPACKET__UNSUBSCRIBED, jid_full(jp->from), "Unsubscribed");
            xmlnode_put_attrib(x, "from", jid_full(jp->to));
            deliver(dpacket_new(x), ti->i);
            return 0;
        /* Do I need to do anything? */
        case JPACKET__SUBSCRIBED:
        case JPACKET__UNSUBSCRIBED:
        default:
            return 0;
    }
    return 0;
}
